@@ -4,5 +4,5 @@ git clean -dfx \
 	-e .settings \
 	-e .metadata \
 	-e Debug/ \
-	-e tal/arin.tal \
+	-e examples/tal/arin.tal \
 	-e tmp
