@@ -51,6 +51,6 @@ int ipv6_prefix_validate(struct ipv6_prefix *);
 bool ipv4_covered(struct in_addr *, uint8_t, struct in_addr *);
 bool ipv6_covered(struct in6_addr *, uint8_t, struct in6_addr *);
 
-char const *sockaddr2str(struct sockaddr_storage *);
+void sockaddr2str(struct sockaddr_storage *, char *buffer);
 
 #endif /* SRC_ADDRESS_H_ */
