@@ -2,6 +2,7 @@
 #define SRC_CERT_STACK_H_
 
 #include <openssl/x509.h>
+#include <stdbool.h>
 #include "resource.h"
 #include "uri.h"
 #include "object/certificate.h"
@@ -47,6 +48,7 @@ void x509stack_cancel(struct cert_stack *);
 X509 *x509stack_peek(struct cert_stack *);
 struct rpki_uri *x509stack_peek_uri(struct cert_stack *);
 struct resources *x509stack_peek_resources(struct cert_stack *);
+unsigned int x509stack_peek_level(struct cert_stack *);
 int x509stack_store_serial(struct cert_stack *, BIGNUM *);
 typedef int (*subject_pk_check_cb)(bool *, char const *, void *);
 int x509stack_store_subject(struct cert_stack *, struct rfc5280_name *,
