@@ -68,22 +68,6 @@ rwlock_unlock(pthread_rwlock_t *lock)
 		    error);
 }
 
-void
-close_thread(pthread_t thread, char const *what)
-{
-	int error;
-
-	error = pthread_cancel(thread);
-	if (error && error != ESRCH)
-		pr_crit("pthread_cancel() threw %d on the '%s' thread.",
-		    error, what);
-
-	error = pthread_join(thread, NULL);
-	if (error)
-		pr_crit("pthread_join() threw %d on the '%s' thread.",
-		    error, what);
-}
-
 static int
 process_file(char const *dir_name, char const *file_name, char const *file_ext,
     int *fcount, process_file_cb cb, void *arg)
@@ -187,9 +171,8 @@ process_file_or_dir(char const *location, char const *file_ext, bool empty_err,
 
 bool
 valid_file_or_dir(char const *location, bool check_file, bool check_dir,
-    int (*cb) (int error, const char *format, ...))
+    int (*error_fn)(int error, const char *format, ...))
 {
-	FILE *file;
 	struct stat attr;
 	bool is_file, is_dir;
 	bool result;
@@ -197,21 +180,12 @@ valid_file_or_dir(char const *location, bool check_file, bool check_dir,
 	if (!check_file && !check_dir)
 		pr_crit("Wrong usage, at least one check must be 'true'.");
 
-	result = false;
-	file = fopen(location, "rb");
-	if (file == NULL) {
-		if (cb != NULL)
-			cb(errno, "Could not open location '%s'",
-			    location);
+	if (stat(location, &attr) == -1) {
+		if (error_fn != NULL)
+			error_fn(errno, "stat(%s) failed", location);
 		return false;
 	}
 
-	if (fstat(fileno(file), &attr) == -1) {
-		if (cb != NULL)
-			cb(errno, "fstat(%s) failed", location);
-		goto end;
-	}
-
 	is_file = check_file && S_ISREG(attr.st_mode);
 	is_dir = check_dir && S_ISDIR(attr.st_mode);
 
@@ -221,10 +195,6 @@ valid_file_or_dir(char const *location, bool check_file, bool check_dir,
 		    (check_file && check_dir) ? "file or directory" :
 		    (check_file) ? "file" : "directory");
 
-end:
-	if (fclose(file) == -1)
-		if (cb != NULL)
-			cb(errno, "fclose() failed");
 	return result;
 }
 
