@@ -40,7 +40,7 @@ struct rpki_config {
 	char *tal;
 	/** Path of our local clone of the repository */
 	char *local_repository;
-	/** FIXME (later) Deprecated, remove it. RSYNC download strategy. */
+	/** TODO (later) Deprecated, remove it. RSYNC download strategy. */
 	enum rsync_strategy sync_strategy;
 	/**
 	 * Handle TAL URIs in random order?
@@ -81,6 +81,8 @@ struct rpki_config {
 			unsigned int retry;
 			unsigned int expire;
 		} interval;
+		/** Number of iterations the deltas will be stored. */
+		unsigned int deltas_lifetime;
 	} server;
 
 	struct {
@@ -192,6 +194,8 @@ struct rpki_config {
 		char *roa;
 		/** File where the validated BGPsec certs will be stored */
 		char *bgpsec;
+		/** Format for the output */
+		enum output_format format;
 	} output;
 
 	/* ASN1 decoder max stack size allowed */
@@ -200,11 +204,12 @@ struct rpki_config {
 	/* Time period that must lapse to warn about a stale repository */
 	unsigned int stale_repository_period;
 
-	/* Download the TALs into --tal? */
+	/* Download the normal TALs into --tal? */
 	bool init_tals;
-
-	/* HTTPS URLS from where the TALS will be fetched */
-	struct init_locations init_tal_locations;
+	/* Download AS0 TALs into --tal? */
+	bool init_tal0s;
+	/* Deprecated; currently does nothing. */
+	unsigned int init_tal_locations;
 
 	/* Thread pools for specific tasks */
 	struct {
@@ -433,6 +438,14 @@ static const struct option_field options[] = {
 		 */
 		.min = 600,
 		.max = 172800,
+	}, {
+		.id = 5007,
+		.name = "server.deltas.lifetime",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, server.deltas_lifetime),
+		.doc = "Number of iterations the deltas will be stored.",
+		.min = 0,
+		.max = UINT_MAX,
 	},
 
 	/* RSYNC fields */
@@ -718,15 +731,21 @@ static const struct option_field options[] = {
 		.name = "output.roa",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, output.roa),
-		.doc = "File where ROAs will be stored in CSV format, use '-' to print at console",
+		.doc = "File where ROAs will be stored, use '-' to print at console",
 		.arg_doc = "<file>",
 	}, {
 		.id = 6001,
 		.name = "output.bgpsec",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, output.bgpsec),
-		.doc = "File where BGPsec Router Keys will be stored in CSV format, use '-' to print at console",
+		.doc = "File where BGPsec Router Keys will be stored, use '-' to print at console",
 		.arg_doc = "<file>",
+	}, {
+		.id = 6002,
+		.name = "output.format",
+		.type = &gt_output_format,
+		.offset = offsetof(struct rpki_config, output.format),
+		.doc = "Format to print ROAs and BGPsec Router Keys",
 	},
 
 	{
@@ -753,15 +772,21 @@ static const struct option_field options[] = {
 		.name = "init-tals",
 		.type = &gt_bool,
 		.offset = offsetof(struct rpki_config, init_tals),
-		.doc = "Fetch the RIR's TAL files into the specified path at --tal",
+		.doc = "Fetch the currently-known TAL files into --tal",
 		.availability = AVAILABILITY_GETOPT,
-	},
-	{
+	}, {
+		.id = 11002,
+		.name = "init-as0-tals",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, init_tal0s),
+		.doc = "Fetch the currently-known AS0 TAL files into --tal",
+		.availability = AVAILABILITY_GETOPT,
+	}, {
 		.id = 11001,
 		.name = "init-locations",
 		.type = &gt_init_tals_locations,
 		.offset = offsetof(struct rpki_config, init_tal_locations),
-		.doc = "Locations from where the TAL files will be downloaded, and its optional accept message",
+		.doc = "Deprecated. Does nothing as of Fort 1.5.1.",
 		.availability = AVAILABILITY_JSON,
 	},
 
@@ -770,10 +795,9 @@ static const struct option_field options[] = {
 		.name = "thread-pool.server.max",
 		.type = &gt_uint,
 		.offset = offsetof(struct rpki_config, thread_pool.server.max),
-		.doc = "Maximum number of active threads (one thread per RTR client) that can live at the thread pool",
+		.doc = "Number of threads in the RTR client request thread pool. Also known as the maximum number of client requests the RTR server will be able to handle at the same time.",
 		.min = 1,
-		/* Would somebody connect more than 500 routers? */
-		.max = 500,
+		.max = UINT_MAX,
 	},
 	{
 		.id = 12001,
@@ -781,8 +805,8 @@ static const struct option_field options[] = {
 		.type = &gt_uint,
 		.offset = offsetof(struct rpki_config,
 		    thread_pool.validation.max),
-		.doc = "Maximum number of active threads (one thread per TAL) that can live at the thread pool",
-		.min = 1,
+		.doc = "Number of threads in the validation thread pool. (Each thread handles one TAL tree.)",
+		.min = 0,
 		.max = 100,
 	},
 
@@ -935,18 +959,6 @@ set_default_values(void)
 		"$LOCAL",
 	};
 
-	static char const *init_locations_no_msg[] = {
-		"https://raw.githubusercontent.com/NICMx/FORT-validator/master/examples/tal/lacnic.tal",
-		"https://raw.githubusercontent.com/NICMx/FORT-validator/master/examples/tal/ripe.tal",
-		"https://raw.githubusercontent.com/NICMx/FORT-validator/master/examples/tal/afrinic.tal",
-		"https://raw.githubusercontent.com/NICMx/FORT-validator/master/examples/tal/apnic.tal",
-	};
-
-	static char const *init_locations_w_msg[] = {
-		"https://www.arin.net/resources/manage/rpki/arin.tal",
-		"Please download and read ARIN Relying Party Agreement (RPA) from https://www.arin.net/resources/manage/rpki/rpa.pdf. Once you've read it and if you agree ARIN RPA, type 'yes' to proceed with ARIN's TAL download:",
-	};
-
 	int error;
 
 	/*
@@ -956,7 +968,7 @@ set_default_values(void)
 
 	error = string_array_init(&rpki_config.server.address, NULL, 0);
 	if (error)
-		return pr_enomem();
+		return error;
 
 	rpki_config.server.port = strdup("323");
 	if (rpki_config.server.port == NULL) {
@@ -969,6 +981,7 @@ set_default_values(void)
 	rpki_config.server.interval.refresh = 3600;
 	rpki_config.server.interval.retry = 600;
 	rpki_config.server.interval.expire = 7200;
+	rpki_config.server.deltas_lifetime = 2;
 
 	rpki_config.tal = NULL;
 	rpki_config.slurm = NULL;
@@ -1023,7 +1036,7 @@ set_default_values(void)
 	rpki_config.http.ca_path = NULL; /* Use system default */
 
 	/*
-	 * FIXME (later) Same values as http.*, delete when rrdp.* is fully
+	 * TODO (later) Same values as http.*, delete when rrdp.* is fully
 	 * deprecated
 	 */
 	rpki_config.rrdp.enabled = rpki_config.http.enabled;
@@ -1058,16 +1071,13 @@ set_default_values(void)
 
 	rpki_config.output.roa = NULL;
 	rpki_config.output.bgpsec = NULL;
+	rpki_config.output.format = OFM_CSV;
 
 	rpki_config.asn1_decode_max_stack = 4096; /* 4kB */
 	rpki_config.stale_repository_period = 43200; /* 12 hours */
 
 	rpki_config.init_tals = false;
-	error = init_locations_init(&rpki_config.init_tal_locations,
-	    init_locations_no_msg, ARRAY_LEN(init_locations_no_msg),
-	    init_locations_w_msg, ARRAY_LEN(init_locations_w_msg));
-	if (error)
-		goto revert_init_locations;
+	rpki_config.init_tal_locations = 0;
 
 	/* Common scenario is to connect 1 router or a couple of them */
 	rpki_config.thread_pool.server.max = 20;
@@ -1075,8 +1085,7 @@ set_default_values(void)
 	rpki_config.thread_pool.validation.max = 5;
 
 	return 0;
-revert_init_locations:
-	free(rpki_config.validation_log.tag);
+
 revert_validation_log_tag:
 	free(rpki_config.http.user_agent);
 revert_flat_array:
@@ -1108,7 +1117,7 @@ validate_config(void)
 
 	/* A file location at --tal isn't valid when --init-tals is set */
 	if (!valid_file_or_dir(rpki_config.tal, !rpki_config.init_tals, true,
-	    pr_op_errno))
+	    __pr_op_err))
 		return pr_op_err("Invalid TAL(s) location.");
 
 	/* Ignore the other checks */
@@ -1130,10 +1139,10 @@ validate_config(void)
 		return pr_op_err("Invalid output.bgpsec file.");
 
 	if (rpki_config.slurm != NULL &&
-	    !valid_file_or_dir(rpki_config.slurm, true, true, pr_op_errno))
+	    !valid_file_or_dir(rpki_config.slurm, true, true, __pr_op_err))
 		return pr_op_err("Invalid slurm location.");
 
-	/* FIXME (later) Remove when sync-strategy is fully deprecated */
+	/* TODO (later) Remove when sync-strategy is fully deprecated */
 	if (!rpki_config.rsync.enabled)
 		config_set_sync_strategy(RSYNC_OFF);
 
@@ -1236,9 +1245,11 @@ handle_flags_config(int argc, char **argv)
 		goto end;
 
 	/* If present, nothing else is done */
-	if (rpki_config.init_tals) {
-		error = init_tals_exec(&rpki_config.init_tal_locations,
-		    rpki_config.tal);
+	if (rpki_config.init_tals || rpki_config.init_tal0s) {
+		if (rpki_config.init_tals)
+			error = download_tals();
+		if (!error && rpki_config.init_tal0s)
+			error = download_tal0s();
 		free(long_opts);
 		free(short_opts);
 		exit(error);
@@ -1299,12 +1310,6 @@ config_get_server_queue(void)
 	return rpki_config.server.backlog;
 }
 
-bool
-config_get_work_offline(void)
-{
-	return rpki_config.work_offline;
-}
-
 unsigned int
 config_get_validation_interval(void)
 {
@@ -1329,6 +1334,12 @@ config_get_interval_expire(void)
 	return rpki_config.server.interval.expire;
 }
 
+unsigned int
+config_get_deltas_lifetime(void)
+{
+	return rpki_config.server.deltas_lifetime;
+}
+
 char const *
 config_get_slurm(void)
 {
@@ -1446,7 +1457,7 @@ config_get_val_log_facility(void)
 bool
 config_get_rsync_enabled(void)
 {
-	return rpki_config.rsync.enabled;
+	return !rpki_config.work_offline && rpki_config.rsync.enabled;
 }
 
 unsigned int
@@ -1501,7 +1512,7 @@ config_get_rsync_args(bool is_ta)
 bool
 config_get_http_enabled(void)
 {
-	return rpki_config.http.enabled;
+	return !rpki_config.work_offline && rpki_config.http.enabled;
 }
 
 unsigned int
@@ -1528,19 +1539,19 @@ config_get_http_user_agent(void)
 	return rpki_config.http.user_agent;
 }
 
-unsigned int
+long
 config_get_http_connect_timeout(void)
 {
 	return rpki_config.http.connect_timeout;
 }
 
-unsigned int
+long
 config_get_http_transfer_timeout(void)
 {
 	return rpki_config.http.transfer_timeout;
 }
 
-unsigned int
+long
 config_get_http_idle_timeout(void)
 {
 	return rpki_config.http.idle_timeout;
@@ -1564,6 +1575,12 @@ config_get_output_bgpsec(void)
 	return rpki_config.output.bgpsec;
 }
 
+enum output_format
+config_get_output_format(void)
+{
+	return rpki_config.output.format;
+}
+
 unsigned int
 config_get_asn1_decode_max_stack(void)
 {
