@@ -7,6 +7,7 @@
 #include "config/filename_format.h"
 #include "config/log_conf.h"
 #include "config/mode.h"
+#include "config/output_format.h"
 #include "config/rsync_strategy.h"
 #include "config/string_array.h"
 #include "config/types.h"
@@ -23,6 +24,7 @@ unsigned int config_get_validation_interval(void);
 unsigned int config_get_interval_refresh(void);
 unsigned int config_get_interval_retry(void);
 unsigned int config_get_interval_expire(void);
+unsigned int config_get_deltas_lifetime(void);
 char const *config_get_slurm(void);
 
 char const *config_get_tal(void);
@@ -30,11 +32,10 @@ char const *config_get_local_repository(void);
 bool config_get_shuffle_tal_uris(void);
 unsigned int config_get_max_cert_depth(void);
 enum mode config_get_mode(void);
-bool config_get_work_offline(void);
 char const *config_get_http_user_agent(void);
-unsigned int config_get_http_connect_timeout(void);
-unsigned int config_get_http_transfer_timeout(void);
-unsigned int config_get_http_idle_timeout(void);
+long config_get_http_connect_timeout(void);
+long config_get_http_transfer_timeout(void);
+long config_get_http_idle_timeout(void);
 char const *config_get_http_ca_path(void);
 bool config_get_rsync_enabled(void);
 unsigned int config_get_rsync_priority(void);
@@ -49,6 +50,7 @@ unsigned int config_get_http_retry_count(void);
 unsigned int config_get_http_retry_interval(void);
 char const *config_get_output_roa(void);
 char const *config_get_output_bgpsec(void);
+enum output_format config_get_output_format(void);
 unsigned int config_get_asn1_decode_max_stack(void);
 unsigned int config_get_stale_repository_period(void);
 unsigned int config_get_thread_pool_server_max(void);
@@ -77,14 +79,14 @@ uint32_t config_get_val_log_facility(void);
  */
 void config_set_rsync_enabled(bool);
 void config_set_http_enabled(bool);
-/* FIXME (later) This will be deprecated */
+/* TODO (later) This will be deprecated */
 void config_set_rrdp_enabled(bool);
 
-/* FIXME (later) Remove when sync-strategy is fully deprecated */
+/* TODO (later) Remove when sync-strategy is fully deprecated */
 void config_set_sync_strategy(enum rsync_strategy);
 void config_set_rsync_strategy(enum rsync_strategy);
 
-/* FIXME (later) Remove once rrdp.* is fully deprecated */
+/* TODO (later) Remove once rrdp.* is fully deprecated */
 void config_set_rrdp_priority(unsigned int);
 void config_set_http_priority(unsigned int);
 void config_set_rrdp_retry_count(unsigned int);
