@@ -69,7 +69,7 @@ traverse(char const *path, struct stat const *sb, int flag, struct FTW *ftwbuf)
 	}
 }
 
-static void *
+static void
 remove_from_root(void *arg)
 {
 	struct rem_dirs *root_arg = arg;
@@ -100,7 +100,6 @@ remove_from_root(void *arg)
 
 	pr_op_debug("Done removing dirs.");
 	free(dirs_arr);
-	return NULL;
 }
 
 /*
@@ -290,7 +289,7 @@ delete_dir_daemon_start(char **roots, size_t roots_len, char const *workspace)
 	}
 
 	/* Thread arg is released at thread */
-	error = internal_pool_push(remove_from_root, (void *) arg);
+	error = internal_pool_push("Directory deleter", remove_from_root, arg);
 	if (error) {
 		rem_dirs_destroy(arg);
 		return error;
