@@ -2,6 +2,7 @@
 
 #include <errno.h>
 #include <stdint.h> /* SIZE_MAX */
+#include <syslog.h>
 #include <time.h>
 #include <openssl/asn1.h>
 #include <sys/socket.h>
@@ -109,7 +110,7 @@ validate_serial_number(X509 *cert)
 	if (number == NULL)
 		return val_crypto_err("Could not parse certificate serial number");
 
-	if (log_val_debug_enabled())
+	if (log_val_enabled(LOG_DEBUG))
 		debug_serial_number(number);
 
 	error = x509stack_store_serial(validation_certstack(state), number);
@@ -1918,7 +1919,7 @@ force_aia_validation(struct rpki_uri *caIssuers, X509 *son)
 
 	/* RSYNC is still the preferred access mechanism, force the sync */
 	do {
-		error = download_files(caIssuers, false, true);
+		error = rsync_download_files(caIssuers, false, true);
 		if (!error)
 			break;
 		if (error == EREQFAILED) {
@@ -2020,7 +2021,7 @@ static int
 verify_mft_loc(struct rpki_uri *mft_uri)
 {
 	if (!valid_file_or_dir(uri_get_local(mft_uri), true, false,
-	    pr_val_errno))
+	    __pr_val_err))
 		return -EINVAL; /* Error already logged */
 
 	return 0;
@@ -2130,7 +2131,7 @@ exec_rsync_method(struct sia_ca_uris *sia_uris)
 
 	/* Stop working on the RRDP local workspace */
 	db_rrdp_uris_workspace_disable();
-	error = download_files(sia_uris->caRepository.uri, false, false);
+	error = rsync_download_files(sia_uris->caRepository.uri, false, false);
 	if (error)
 		return error;
 
@@ -2555,7 +2556,7 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 		 */
 		pr_val_info("Retrying repository download to discard 'transient inconsistency' manifest issue (see RFC 6481 section 5) '%s'",
 		    uri_val_get_printable(sia_uris.caRepository.uri));
-		error = download_files(sia_uris.caRepository.uri, false, true);
+		error = rsync_download_files(sia_uris.caRepository.uri, false, true);
 		if (error)
 			break;
 
