@@ -6,7 +6,7 @@
 #include "uri.h"
 #include "visited_uris.h"
 
-int rrdp_parse_notification(struct rpki_uri *, bool,
+int rrdp_parse_notification(struct rpki_uri *, bool, bool,
     struct update_notification **);
 int rrdp_parse_snapshot(struct update_notification *, struct visited_uris *,
     bool);
