@@ -4,6 +4,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h> /* SIGINT, SIGQUIT, etc */
+#include <syslog.h>
 #include <sys/queue.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -164,7 +165,7 @@ get_rsync_uri(struct rpki_uri *requested_uri, bool is_ta,
 		    : handle_root_strategy(requested_uri, rsync_uri);
 	case RSYNC_STRICT:
 		return handle_strict_strategy(requested_uri, rsync_uri);
-	default:
+	case RSYNC_OFF:
 		break;
 	}
 
@@ -243,7 +244,7 @@ prepare_rsync(struct rpki_uri *uri, bool is_ta, char ***args, size_t *args_len)
 	return 0;
 }
 
-static void
+__dead static void
 handle_child_thread(char **args, int fds[2][2])
 {
 	/* THIS FUNCTION MUST NEVER RETURN!!! */
@@ -275,17 +276,16 @@ create_pipes(int fds[2][2])
 	return 0;
 }
 
-static void
+static int
 log_buffer(char const *buffer, ssize_t read, int type, bool log_operation)
 {
 #define PRE_RSYNC "[RSYNC exec]: "
 	char *cpy, *cur, *tmp;
 
 	cpy = malloc(read + 1);
-	if (cpy == NULL) {
-		pr_enomem();
-		return;
-	}
+	if (cpy == NULL)
+		return pr_enomem();
+
 	strncpy(cpy, buffer, read);
 	cpy[read] = '\0';
 
@@ -307,6 +307,7 @@ log_buffer(char const *buffer, ssize_t read, int type, bool log_operation)
 		cur = tmp + 1;
 	}
 	free(cpy);
+	return 0;
 #undef PRE_RSYNC
 }
 
@@ -315,6 +316,7 @@ read_pipe(int fd_pipe[2][2], int type, bool log_operation)
 {
 	char buffer[4096];
 	ssize_t count;
+	int error;
 
 	while (1) {
 		count = read(fd_pipe[type][0], buffer, sizeof(buffer));
@@ -327,7 +329,9 @@ read_pipe(int fd_pipe[2][2], int type, bool log_operation)
 		if (count == 0)
 			break;
 
-		log_buffer(buffer, count, type, log_operation);
+		error = log_buffer(buffer, count, type, log_operation);
+		if (error)
+			return error;
 	}
 	close(fd_pipe[type][0]); /* Close read end */
 	return 0;
@@ -381,9 +385,11 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 	if (error)
 		return error;
 
-	pr_val_debug("Executing RSYNC:");
-	for (i = 0; i < args_len + 1; i++)
-		pr_val_debug("    %s", args[i]);
+	if (log_val_enabled(LOG_DEBUG)) {
+		pr_val_debug("Executing RSYNC:");
+		for (i = 0; i < args_len + 1; i++)
+			pr_val_debug("    %s", args[i]);
+	}
 
 	retries = 0;
 	do {
@@ -447,9 +453,6 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 			error = WEXITSTATUS(child_status);
 			pr_val_debug("Child terminated with error code %d.",
 			    error);
-			if (error == -ENOMEM)
-				pr_enomem();
-
 			if (!error)
 				goto release_args;
 
@@ -558,7 +561,7 @@ check_ancestor_error(struct rpki_uri *requested_uri)
  * validated the TA's public key.
  */
 int
-download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
+rsync_download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 {
 	/**
 	 * Note:
@@ -587,9 +590,9 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 		return check_ancestor_error(requested_uri);
 	}
 
-	if (!force)
+	if (!force) {
 		error = get_rsync_uri(requested_uri, is_ta, &rsync_uri);
-	else {
+	} else {
 		error = check_ancestor_error(requested_uri);
 		if (error)
 			return error;
@@ -620,8 +623,6 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 		if (!error)
 			error = EREQFAILED;
 		break;
-	default:
-		break;
 	}
 
 	uri_refput(rsync_uri);
