@@ -2,28 +2,21 @@
 
 #include <pthread.h>
 #include <string.h>
+#include <syslog.h>
 #include <time.h>
 #include <sys/queue.h>
-#include "clients.h"
+
 #include "common.h"
 #include "output_printer.h"
 #include "validation_handler.h"
 #include "data_structure/array_list.h"
 #include "object/router_key.h"
 #include "object/tal.h"
+#include "rtr/rtr.h"
 #include "rtr/db/db_table.h"
 #include "slurm/slurm_loader.h"
 #include "thread/thread_pool.h"
 
-/*
- * Storage of VRPs (term taken from RFC 6811 "Validated ROA Payload") and
- * Serials that contain such VRPs
- */
-
-#define START_SERIAL		0
-
-DEFINE_ARRAY_LIST_FUNCTIONS(deltas_db, struct delta_group, )
-
 struct vrp_node {
 	struct delta_vrp delta;
 	SLIST_ENTRY(vrp_node) next;
@@ -53,12 +46,32 @@ struct state {
 	 */
 	struct db_table *base;
 	/** DB changes to @base over time. */
-	struct deltas_db deltas;
+	struct deltas_array *deltas;
 
-	/* Last valid SLURM applied to base */
+	/*
+	 * Last valid SLURM applied to base.
+	 *
+	 * Doesn't need locking, because the only writer is also the only
+	 * reader.
+	 */
 	struct db_slurm *slurm;
 
-	serial_t next_serial;
+	/*
+	 * This is the serial number of base.
+	 *
+	 * At least one RTR client implementation (Cloudflare's rpki-rtr-client)
+	 * malfunctions if the validator uses zero as the first serial, so this
+	 * value behaves as follows:
+	 *
+	 * serial = 0. After every successful validation cycle, serial++.
+	 *
+	 * Do not use this value to check whether we already finished our first
+	 * validation. (Use base != NULL for that.) Zero is totally a valid
+	 * serial, particularly when the integer wraps.
+	 *
+	 * TODO (fine) this should probably be moved to struct db_table.
+	 */
+	serial_t serial;
 	uint16_t v0_session_id;
 	uint16_t v1_session_id;
 };
@@ -68,18 +81,12 @@ static struct state state;
 /* Thread pool to use when the TALs will be processed */
 static struct thread_pool *pool;
 
-/** Read/write lock, which protects @state and its inhabitants. */
+/** Protects @state.base, @state.deltas and @state.serial. */
 static pthread_rwlock_t state_lock;
 
 /** Lock to protect ROA table during construction. */
 static pthread_rwlock_t table_lock;
 
-void
-deltagroup_cleanup(struct delta_group *group)
-{
-	deltas_refput(group->deltas);
-}
-
 int
 vrps_init(void)
 {
@@ -87,27 +94,30 @@ vrps_init(void)
 	int error;
 
 	pool = NULL;
-	error = thread_pool_create(config_get_thread_pool_validation_max(),
-	    &pool);
+	error = thread_pool_create("Validation",
+	    config_get_thread_pool_validation_max(), &pool);
 	if (error)
 		return error;
 
 	state.base = NULL;
-
-	deltas_db_init(&state.deltas);
+	state.deltas = darray_create();
+	if (state.deltas == NULL) {
+		error = pr_enomem();
+		goto revert_thread_pool;
+	}
 
 	/*
 	 * Use the same start serial, the session ID will avoid
 	 * "desynchronization" (more at RFC 6810 'Glossary' and
 	 * 'Fields of a PDU')
 	 */
-	state.next_serial = START_SERIAL;
+	state.serial = 0;
 
 	/* Get the bits that'll fit in session_id */
 	now = 0;
 	error = get_current_time(&now);
 	if (error)
-		goto release_deltas;
+		goto revert_deltas;
 	state.v0_session_id = now & 0xFFFF;
 
 	/* Minus 1 to prevent same ID */
@@ -120,20 +130,22 @@ vrps_init(void)
 	error = pthread_rwlock_init(&state_lock, NULL);
 	if (error) {
 		error = pr_op_errno(error, "state pthread_rwlock_init() errored");
-		goto release_deltas;
+		goto revert_deltas;
 	}
 
 	error = pthread_rwlock_init(&table_lock, NULL);
 	if (error) {
 		error = pr_op_errno(error, "table pthread_rwlock_init() errored");
-		goto release_state_lock;
+		goto revert_state_lock;
 	}
 
 	return 0;
-release_state_lock:
+
+revert_state_lock:
 	pthread_rwlock_destroy(&state_lock);
-release_deltas:
-	deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
+revert_deltas:
+	darray_destroy(state.deltas);
+revert_thread_pool:
 	thread_pool_destroy(pool);
 	return error;
 }
@@ -141,15 +153,17 @@ release_deltas:
 void
 vrps_destroy(void)
 {
-	if (state.base != NULL)
-		db_table_destroy(state.base);
-	if (state.slurm != NULL)
-		db_slurm_destroy(state.slurm);
-	deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
-	/* Nothing to do with error codes from now on */
+	thread_pool_destroy(pool);
+
 	pthread_rwlock_destroy(&state_lock);
 	pthread_rwlock_destroy(&table_lock);
-	thread_pool_destroy(pool);
+
+	if (state.slurm != NULL)
+		db_slurm_destroy(state.slurm);
+
+	darray_destroy(state.deltas);
+	if (state.base != NULL)
+		db_table_destroy(state.base);
 }
 
 #define WLOCK_HANDLER(lock, cb)						\
@@ -210,181 +224,105 @@ __perform_standalone_validation(struct db_table **result)
 	return 0;
 }
 
-/*
- * Make an empty dummy delta array.
- * It's annoying, but temporary. (Until it expires.) Otherwise, it'd be a pain
- * to have to check NULL delta_group.deltas all the time.
- */
 static int
-create_empty_delta(struct deltas **deltas)
+__compute_deltas(struct db_table *old_base, struct db_table *new_base,
+    bool *notify_clients, struct deltas **result)
 {
-	struct delta_group deltas_node;
 	int error;
 
-	error = deltas_create(deltas);
+	*result = NULL;
+	if (notify_clients != NULL)
+		*notify_clients = true;
+
+	/* First version of the database = No deltas */
+	if (old_base == NULL)
+		return 0;
+
+	error = compute_deltas(old_base, new_base, result);
 	if (error)
 		return error;
 
-	deltas_node.serial = state.next_serial;
-	deltas_node.deltas = *deltas;
-	error = deltas_db_add(&state.deltas, &deltas_node);
-	if (error)
-		deltas_refput(*deltas);
-	return error;
-}
-
-/**
- * Reallocate the array of @db starting at @start, the length and capacity are
- * calculated according to the new start.
- */
-static int
-resize_deltas_db(struct deltas_db *db, struct delta_group *start)
-{
-	struct delta_group *tmp, *ptr;
-
-	db->len -= (start - db->array);
-	while (db->len < db->capacity / 2)
-		db->capacity /= 2;
-	tmp = malloc(sizeof(struct delta_group) * db->capacity);
-	if (tmp == NULL)
-		return pr_enomem();
-
-	memcpy(tmp, start, db->len * sizeof(struct delta_group));
-	/* Release memory allocated */
-	for (ptr = db->array; ptr < start; ptr++)
-		deltas_refput(ptr->deltas);
-	free(db->array);
-	db->array = tmp;
+	if (deltas_is_empty(*result)) {
+		if (notify_clients != NULL)
+			*notify_clients = false;
+		deltas_refput(*result);
+		*result = NULL;
+	}
 
 	return 0;
 }
 
-/*
- * Lock must be requested before calling this function
- */
 static int
-vrps_purge(struct deltas **deltas)
+__vrps_update(bool *notify_clients)
 {
-	struct delta_group *group;
-	array_index i;
-	serial_t min_serial;
+	/*
+	 * This function is the only writer, and it runs once at a time.
+	 * Therefore, it's going to worry about write locking, but not read
+	 * locking.
+	 */
 
-	if (clients_get_min_serial(&min_serial) != 0) {
-		/* Nobody will need deltas, just leave an empty one */
-		deltas_refput(*deltas);
-		deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
-		deltas_db_init(&state.deltas);
-		return create_empty_delta(deltas);
-	}
-
-	/* Assume its ordered by serial, so get the new initial pointer */
-	ARRAYLIST_FOREACH(&state.deltas, group, i)
-		if (group->serial >= min_serial)
-			break;
-
-	/* Its the first element or reached end, nothing to purge */
-	if (group == state.deltas.array ||
-	    (group - state.deltas.array) == state.deltas.len)
-		return 0;
-
-	return resize_deltas_db(&state.deltas, group);
-}
-
-static int
-__vrps_update(bool *changed)
-{
 	struct db_table *old_base;
 	struct db_table *new_base;
-	struct deltas *deltas; /* Deltas in raw form */
-	struct delta_group deltas_node; /* Deltas in database node form */
-	serial_t min_serial;
+	struct deltas *new_deltas;
 	int error;
 
-	*changed = false;
-	old_base = NULL;
+	if (notify_clients)
+		*notify_clients = false;
+	old_base = state.base;
 	new_base = NULL;
 
 	error = __perform_standalone_validation(&new_base);
 	if (error)
 		return error;
-
 	error = slurm_apply(&new_base, &state.slurm);
-	if (error)
-		goto revert_base;
+	if (error) {
+		db_table_destroy(new_base);
+		return error;
+	}
+
+	/*
+	 * At this point, new_base is completely valid. Even if we error out
+	 * later, report the ROAs.
+	 *
+	 * This is done after the validation, not during it, to prevent
+	 * duplicate ROAs.
+	 */
+	output_print_data(new_base);
+
+	error = __compute_deltas(old_base, new_base, notify_clients,
+	    &new_deltas);
+	if (error) {
+		/*
+		 * Deltas are nice-to haves. As long as state.base is correct,
+		 * the validator can continue serving the routers.
+		 * (Albeit less efficiently.)
+		 * So drop a warning and keep going.
+		 */
+		pr_op_warn("Deltas could not be computed: %s", strerror(error));
+	}
 
 	rwlock_write_lock(&state_lock);
 
-	if (state.base != NULL) {
-		error = compute_deltas(state.base, new_base, &deltas);
-		if (error) {
-			rwlock_unlock(&state_lock);
-			goto revert_base;
-		}
-
-		if (deltas_is_empty(deltas)) {
-			rwlock_unlock(&state_lock);
-			goto revert_deltas; /* error == 0 is good */
-		}
-
-		/* Just store deltas if someone will care about it */
-		if (clients_get_min_serial(&min_serial) == 0) {
-			deltas_node.serial = state.next_serial;
-			deltas_node.deltas = deltas;
-			error = deltas_db_add(&state.deltas, &deltas_node);
-			if (error) {
-				rwlock_unlock(&state_lock);
-				goto revert_deltas;
-			}
-		}
-
-		/*
-		 * Postpone destruction of the old database,
-		 * to release the lock ASAP.
-		 */
-		old_base = state.base;
-
-		/* Remove unnecessary deltas */
-		error = vrps_purge(&deltas);
-		if (error) {
-			rwlock_unlock(&state_lock);
-			goto revert_base;
-		}
-	} else {
-		/* There's also an empty base, don't alter state */
-		if (db_table_roa_count(new_base) +
-		    db_table_router_key_count(new_base) == 0) {
-			rwlock_unlock(&state_lock);
-			error = 0; /* OK (said explicitly) */
-			goto revert_base;
-		}
-		error = create_empty_delta(&deltas);
-		if (error) {
-			rwlock_unlock(&state_lock);
-			goto revert_base;
-		}
-	}
-
-	*changed = true;
 	state.base = new_base;
-	state.next_serial++;
+	state.serial++;
+	if (new_deltas != NULL) {
+		/* Ownership transferred */
+		darray_add(state.deltas, new_deltas);
+	} else {
+		/*
+		 * If the latest base has no deltas, all existing deltas are
+		 * rendered useless. This is because clients always want to
+		 * reach the latest serial, no matter where they are.
+		 */
+		darray_clear(state.deltas);
+	}
 
 	rwlock_unlock(&state_lock);
 
 	if (old_base != NULL)
 		db_table_destroy(old_base);
 
-	/* Print after validation to avoid duplicated info */
-	output_print_data(new_base);
-
 	return 0;
-
-revert_deltas:
-	deltas_refput(deltas);
-revert_base:
-	/* Print info that was already validated */
-	output_print_data(new_base);
-	db_table_destroy(new_base);
-	return error;
 }
 
 int
@@ -396,18 +334,17 @@ vrps_update(bool *changed)
 	int error;
 
 	/*
-	 * This wrapper is mainly for log informational data, so if there's no
-	 * need don't do unnecessary calls
+	 * This wrapper is mainly intended to log informational data, so if
+	 * there's no need, don't do unnecessary calls.
 	 */
-	if (!log_op_info_enabled())
+	if (!log_op_enabled(LOG_INFO))
 		return __vrps_update(changed);
 
 	pr_op_info("Starting validation.");
-	serial = START_SERIAL;
 	if (config_get_mode() == SERVER) {
 		error = get_last_serial_number(&serial);
 		if (!error)
-			pr_op_info("- Current serial number is %u.", serial);
+			pr_op_info("- Serial before validation: %u", serial);
 	}
 
 	time(&start);
@@ -430,11 +367,8 @@ vrps_update(bool *changed)
 		pr_op_info("- Valid Prefixes: %u", db_table_roa_count(state.base));
 		pr_op_info("- Valid Router Keys: %u",
 		    db_table_router_key_count(state.base));
-		if (config_get_mode() == SERVER) {
-			pr_op_info("- %s serial number is %u.",
-			    serial == state.next_serial - 1 ? "Current" : "New",
-			    state.next_serial - 1);
-		}
+		if (config_get_mode() == SERVER)
+			pr_op_info("- Serial: %u", state.serial);
 		rwlock_unlock(&state_lock);
 	} while(0);
 	pr_op_info("- Real execution time: %ld secs.", exec_time);
@@ -533,21 +467,64 @@ router_key_ovrd_remove(struct delta_router_key const *delta, void *arg)
 	return 0;
 }
 
-/*
- * Remove all operations on @deltas that override each other, and do @cb (with
- * @arg) on each element of the resultant delta.
+static int
+__deltas_foreach(struct deltas *deltas, void *arg)
+{
+	return deltas_foreach(deltas, vrp_ovrd_remove, router_key_ovrd_remove,
+	    arg);
+}
+
+/**
+ * Runs @vrp_cb and @rk_cb on all the deltas from the database whose
+ * serial > @from, excluding those that cancel each other.
+ *
+ * Please keep in mind that there is at least one errcode-aware caller. The most
+ * important ones are
+ * 1. 0: No errors.
+ * 2. -EAGAIN: No data available; database still under construction.
+ * 3. -ESRCH: @from was not found.
  */
 int
-vrps_foreach_filtered_delta(struct deltas_db *deltas,
-    delta_vrp_foreach_cb cb_prefix, delta_router_key_foreach_cb cb_rk,
+vrps_foreach_delta_since(serial_t from, serial_t *to,
+    delta_vrp_foreach_cb vrp_cb, delta_router_key_foreach_cb rk_cb,
     void *arg)
 {
 	struct sorted_lists filtered_lists;
-	struct delta_group *group;
 	struct vrp_node *vnode;
 	struct rk_node *rnode;
-	array_index i;
-	int error = 0;
+	int error;
+
+	error = rwlock_read_lock(&state_lock);
+	if (error)
+		return error;
+
+	if (state.base == NULL) {
+		/* Database still under construction. */
+		rwlock_unlock(&state_lock);
+		return -EAGAIN;
+	}
+
+	if (from == state.serial) {
+		/* Client already has the latest serial. */
+		rwlock_unlock(&state_lock);
+		*to = from;
+		return 0;
+	}
+
+	/* if from < first serial */
+	if (serial_lt(from, state.serial - darray_len(state.deltas)))
+		goto cache_reset; /* Delta was already deleted. */
+	/* if from > last serial */
+	if (serial_lt(state.serial, from))
+		goto cache_reset; /* Serial is invalid. */
+
+	/*
+	 * TODO (performance) this implementation is naive.
+	 * Either use a hash set, or sort the resources.
+	 * Also, deltas that share a serial do not need to be compared to each
+	 * other. (Corollary: If there's one serial, no comparisons whatsoever
+	 * need to be made.)
+	 */
 
 	/*
 	 * Filter: Remove entries that cancel each other.
@@ -556,21 +533,20 @@ vrps_foreach_filtered_delta(struct deltas_db *deltas,
 	 */
 	SLIST_INIT(&filtered_lists.prefixes);
 	SLIST_INIT(&filtered_lists.router_keys);
-	ARRAYLIST_FOREACH(deltas, group, i) {
-		error = deltas_foreach(group->serial, group->deltas,
-		    vrp_ovrd_remove, router_key_ovrd_remove, &filtered_lists);
-		if (error)
-			goto release_list;
-	}
+
+	error = darray_foreach_since(state.deltas, state.serial - from,
+	    __deltas_foreach, &filtered_lists);
+	if (error)
+		goto release_list;
 
 	/* Now do the corresponding callback on the filtered deltas */
 	SLIST_FOREACH(vnode, &filtered_lists.prefixes, next) {
-		error = cb_prefix(&vnode->delta, arg);
+		error = vrp_cb(&vnode->delta, arg);
 		if (error)
 			break;
 	}
 	SLIST_FOREACH(rnode, &filtered_lists.router_keys, next) {
-		error = cb_rk(&rnode->delta, arg);
+		error = rk_cb(&rnode->delta, arg);
 		if (error)
 			break;
 	}
@@ -587,62 +563,13 @@ release_list:
 		free(rnode);
 	}
 
-	return error;
-}
-
-/**
- * Adds to @result the deltas whose serial > @from.
- *
- * Please keep in mind that there is at least one errcode-aware caller. The most
- * important ones are
- * 1. 0: No errors.
- * 2. -EAGAIN: No data available; database still under construction.
- * 3. -ESRCH: @from was not found.
- *
- * As usual, only 0 guarantees valid out parameters. (@to and @result.)
- * (But note that @result is supposed to be already initialized, so caller will
- * have to clean it up regardless of error.)
- */
-int
-vrps_get_deltas_from(serial_t from, serial_t *to, struct deltas_db *result)
-{
-	struct delta_group *group;
-	array_index i;
-	bool from_found;
-	int error;
-
-	from_found = false;
-
-	error = rwlock_read_lock(&state_lock);
-	if (error)
-		return error;
-
-	if (state.base == NULL) {
-		rwlock_unlock(&state_lock);
-		return -EAGAIN;
-	}
-
-	ARRAYLIST_FOREACH(&state.deltas, group, i) {
-		if (!from_found) {
-			if (group->serial == from) {
-				from_found = true;
-				*to = group->serial;
-			}
-			continue;
-		}
-
-		error = deltas_db_add(result, group);
-		if (error) {
-			rwlock_unlock(&state_lock);
-			return error;
-		}
-
-		deltas_refget(group->deltas);
-		*to = group->serial;
-	}
-
+	*to = state.serial;
 	rwlock_unlock(&state_lock);
-	return from_found ? 0 : -ESRCH;
+	return 0;
+
+cache_reset:
+	rwlock_unlock(&state_lock);
+	return -ESRCH;
 }
 
 int
@@ -655,7 +582,7 @@ get_last_serial_number(serial_t *result)
 		return error;
 
 	if (state.base != NULL)
-		*result = state.next_serial - 1;
+		*result = state.serial;
 	else
 		error = -EAGAIN;
 
@@ -667,7 +594,10 @@ get_last_serial_number(serial_t *result)
 uint16_t
 get_current_session_id(uint8_t rtr_version)
 {
-	/* Semaphore isn't needed since this value is set at initialization */
+	/*
+	 * These values are constant after initialization,
+	 * so locking isn't needed.
+	 */
 	if (rtr_version == 1)
 		return state.v1_session_id;
 	return state.v0_session_id;
