@@ -3,9 +3,9 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 
 #include "address.h"
-#include "clients.h"
 #include "common.h"
 #include "log.h"
 #include "rtr/err_pdu.h"
@@ -43,111 +43,106 @@ pdutype2str(enum pdu_type type)
 static int
 pdu_header_from_reader(struct pdu_reader *reader, struct pdu_header *header)
 {
-	return read_int8(reader, &header->protocol_version)
-	    || read_int8(reader, &header->pdu_type)
-	    || read_int16(reader, &header->m.session_id)
-	    || read_int32(reader, &header->length);
-}
-
-/* Do not use this macro before @header has been initialized, obviously. */
-#define RESPOND_ERROR(report_cb) \
-	((header.pdu_type != PDU_TYPE_ERROR_REPORT) ? (report_cb) : -EINVAL);
-
-static int
-validate_rtr_version(int fd, struct pdu_header *header,
-    unsigned char *hdr_bytes)
-{
-	uint8_t curr_version;
-	bool version_set;
 	int error;
 
-	error = clients_get_rtr_version_set(fd, &version_set, &curr_version);
+	error = read_int8(reader, &header->protocol_version);
 	if (error)
-		/* Unlikely, but close connection with min version */
-		return (header->pdu_type != PDU_TYPE_ERROR_REPORT) ?
-		    err_pdu_send_internal_error(fd, RTR_V0) : -EINVAL;
+		return error;
+	error = read_int8(reader, &header->pdu_type);
+	if (error)
+		return error;
+	error = read_int16(reader, &header->m.session_id);
+	if (error)
+		return error;
+	return read_int32(reader, &header->length);
+}
 
-	if (version_set) {
-		if (header->protocol_version == curr_version)
+static int
+validate_rtr_version(struct rtr_client *client, struct pdu_header *header,
+    unsigned char *hdr_bytes)
+{
+	if (client->rtr_version != -1) {
+		if (header->protocol_version == client->rtr_version)
 			return 0;
 
 		/* Don't send error on a rcvd error! */
 		if (header->pdu_type == PDU_TYPE_ERROR_REPORT)
 			return -EINVAL;
 
-		switch (curr_version) {
+		switch (client->rtr_version) {
 		case RTR_V1:
 			/* Rcvd version is valid, but unexpected */
 			if (header->protocol_version == RTR_V0)
 				return err_pdu_send_unexpected_proto_version(
-				    fd, curr_version, hdr_bytes,
+				    client->fd, client->rtr_version, hdr_bytes,
 				    "RTR version 0 was expected");
 			/* Send common error */
 		case RTR_V0:
-			return err_pdu_send_unsupported_proto_version(fd,
-			    curr_version, hdr_bytes,
+			return err_pdu_send_unsupported_proto_version(
+			    client->fd, client->rtr_version, hdr_bytes,
 			    "RTR version received is unknown.");
 		default:
-			pr_crit("Unknown RTR version %u", curr_version);
-			break;
+			pr_crit("Unknown RTR version %u", client->rtr_version);
 		}
 	}
 
 	/* Unsigned and incremental values, so compare against major version */
 	if (header->protocol_version > RTR_V1)
 		/* ...and send error with min version */
-		return (header->pdu_type != PDU_TYPE_ERROR_REPORT) ?
-		    err_pdu_send_unsupported_proto_version(fd, RTR_V0,
-		    hdr_bytes, "RTR version received is unknown.")
+		return (header->pdu_type != PDU_TYPE_ERROR_REPORT)
+		    ? err_pdu_send_unsupported_proto_version(client->fd, RTR_V0,
+		          hdr_bytes, "RTR version received is unknown.")
 		    : -EINVAL;
 
-	return clients_set_rtr_version(fd, header->protocol_version);
+	client->rtr_version = header->protocol_version;
+	return 0;
 }
 
+/* Do not use this macro before @header has been initialized, obviously. */
+#define RESPOND_ERROR(report_cb) \
+	((header.pdu_type != PDU_TYPE_ERROR_REPORT) ? (report_cb) : -EINVAL);
+
+/*
+ * Reads the next PDU from @reader. Returns the PDU in @request, and its
+ * metadata in @metadata.
+ */
 int
-pdu_load(int fd, struct sockaddr_storage *client_addr,
+pdu_load(struct pdu_reader *reader, struct rtr_client *client,
     struct rtr_request *request, struct pdu_metadata const **metadata)
 {
-	unsigned char hdr_bytes[RTRPDU_HDR_LEN];
-	struct pdu_reader reader;
 	struct pdu_header header;
 	struct pdu_metadata const *meta;
-	uint8_t version;
 	int error;
 
-	/* Read the header into its buffer. */
-	error = pdu_reader_init(&reader, fd, hdr_bytes, RTRPDU_HDR_LEN, true);
-	if (error)
-		/* Communication interrupted; omit error response */
-		return error;
-	error = pdu_header_from_reader(&reader, &header);
+	if (reader->size == 0) {
+		pr_op_debug("Client packet contains no more PDUs.");
+		return ENOENT;
+	}
+
+	request->bytes = reader->buffer;
+	request->bytes_len = RTRPDU_HDR_LEN;
+
+	error = pdu_header_from_reader(reader, &header);
 	if (error)
 		/* No error response because the PDU might have been an error */
 		return error;
 
+	pr_op_debug("PDU '%s' received from client '%s'",
+	    pdutype2str(header.pdu_type), client->addr);
 
-	if (log_op_debug_enabled())
-		pr_op_debug("Received a %s from %s.",
-		    pdutype2str(header.pdu_type),
-		    sockaddr2str(client_addr));
-
-	error = validate_rtr_version(fd, &header, hdr_bytes);
+	error = validate_rtr_version(client, &header, request->bytes);
 	if (error)
 		return error; /* Error response PDU already sent */
 
-	/* RTR version ok, keep it for later use */
-	version = header.protocol_version;
-
 	/*
 	 * DO NOT USE THE err_pdu_* functions directly. Wrap them with
 	 * RESPOND_ERROR() INSTEAD.
-	 * TODO I think this comment should be above validate_rtr_version(),
-	 * and validate_rtr_version() is buggy.
 	 */
 
 	if (header.length < RTRPDU_HDR_LEN)
-		return RESPOND_ERROR(err_pdu_send_invalid_request_truncated(fd,
-		    version, hdr_bytes, "Invalid header length. (< 8 bytes)"));
+		return RESPOND_ERROR(err_pdu_send_invalid_request_truncated(
+		    client->fd, client->rtr_version, request->bytes,
+		    "Invalid header length. (< 8 bytes)"));
 
 	/*
 	 * Error messages can be quite large.
@@ -157,60 +152,33 @@ pdu_load(int fd, struct sockaddr_storage *client_addr,
 	 * (Warning: I'm assuming english tho.)
 	 */
 	if (header.length > 512)
-		return RESPOND_ERROR(err_pdu_send_invalid_request_truncated(fd,
-		    version, hdr_bytes, "PDU is too large. (> 512 bytes)"));
+		return RESPOND_ERROR(err_pdu_send_invalid_request_truncated(
+		    client->fd, client->rtr_version, request->bytes,
+		    "PDU is too large. (> 512 bytes)"));
 
-	/* Read the rest of the PDU into its buffer. */
 	request->bytes_len = header.length;
-	request->bytes = malloc(header.length);
-	if (request->bytes == NULL)
-		/* No error report PDU on allocation failures. */
-		return pr_enomem();
-
-	memcpy(request->bytes, hdr_bytes, RTRPDU_HDR_LEN);
-	error = pdu_reader_init(&reader, fd,
-	    request->bytes + RTRPDU_HDR_LEN,
-	    header.length - RTRPDU_HDR_LEN,
-	    false);
-	if (error)
-		/* Communication interrupted; no error PDU. */
-		goto revert_bytes;
 
 	/* Deserialize the PDU. */
 	meta = pdu_get_metadata(header.pdu_type);
-	if (!meta) {
-		error = RESPOND_ERROR(err_pdu_send_unsupported_pdu_type(fd,
-		    version, request));
-		goto revert_bytes;
-	}
+	if (!meta)
+		return RESPOND_ERROR(err_pdu_send_unsupported_pdu_type(
+		    client->fd, client->rtr_version, request));
 
 	request->pdu = malloc(meta->length);
-	if (request->pdu == NULL) {
-		error = pr_enomem();
-		goto revert_bytes;
-	}
+	if (request->pdu == NULL)
+		/* No error report PDU on allocation failures. */
+		return pr_enomem();
 
-	error = meta->from_stream(&header, &reader, request->pdu);
-	if (reader.size != 0) {
-		error = RESPOND_ERROR(err_pdu_send_invalid_request(fd, version,
-		    request,
-		    "The PDU length sent doesn't match the real PDU length"));
-		goto revert_pdu;
-	}
+	error = meta->from_stream(&header, reader, request->pdu);
 	if (error) {
-		RESPOND_ERROR(err_pdu_send_internal_error(fd, version));
-		goto revert_pdu;
+		/* Communication interrupted; no error PDU. */
+		free(request->pdu);
+		return error;
 	}
 
 	/* Happy path. */
 	*metadata = meta;
 	return 0;
-
-revert_pdu:
-	free(request->pdu);
-revert_bytes:
-	free(request->bytes);
-	return error;
 }
 
 static int
@@ -343,7 +311,7 @@ error_report_destroy(void *pdu_void)
 	}
 
 DEFINE_METADATA(serial_notify, free);
-DEFINE_METADATA(serial_query, free);
+DEFINE_METADATA(serial_query, free); /* handle_serial_query_pdu */
 DEFINE_METADATA(reset_query, free);
 DEFINE_METADATA(cache_response, free);
 DEFINE_METADATA(ipv4_prefix, free);
